/*********************************************************************
 * Licensed Materials - PropertyImpl of IBM                              *
 *                                                                   *
 * 5724-B88                                                          *
 *                                                                   *
 *  Copyright IBM Corp.  2001, 2002    All Rights Reserved.         *
 *                                                                   *
 * US Government Users Restricted Rights - Use, duplication or       *
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp. *
 *********************************************************************/

package com.ibm.websphere.query.base;


/**
	<P>The OrderByExpression class represents an order by expression in an order by clause in the 
	query framework.
	
	<P>This class contains an order by element and an operator to describe a typical order by 
	expression in an order by clause. For example, "order by age in ascending order" will be 
	decomposed as order by element = Attribute("age"), operator = Operator("ASC")
 */
public class OrderByExpression implements IOrderByExpression
{
  private IOrderByElement orderByElement = null;
  private IOperator operator = null;

  /**
  	Creates a new OrderByExpression instance, default constructor.
   */
  public OrderByExpression()
  {
    super();
  }
  /**
  	Creates a new OrderByExpression instance with the given operator and order by element.
   */
  public OrderByExpression(IOperator newOperator, IOrderByElement newOrderByElement)
  {
    super();
    operator = newOperator;
    orderByElement = newOrderByElement;
  }
  /**
  	Builds a query string for the condition.
   */
  public String buildString(ISelectQueryCallback builder) throws QueryException
  {
    return builder.buildOrderByExpression(this);
  }
  public void buildString(ISelectQueryCallback builder, StringBuffer sb) throws QueryException
  {
    builder.buildOrderByExpression(this, sb);
  }
  /**
  	Tests for equality with the given object.
   */
  public boolean equals(Object o)
  {
    boolean eq = true;

    if (o instanceof OrderByExpression)
    {
      OrderByExpression ob = (OrderByExpression) o;

      if (operator != null)
      {
        eq = operator.equals(ob.operator);
      }
      else
      {
        eq = (ob.operator == null);
      }

      if (eq)
      {
        if (orderByElement != null)
        {
          eq = orderByElement.equals(ob.orderByElement);
        }
        else
        {
          eq = (ob.orderByElement == null);
        }
      }
    }
    else
    {
      eq = false;
    }

    return eq;
  }
  /**
  	Returns the operator.
   */
  public IOperator getOperator()
  {
    return operator;
  }
  /**
  	Returns the order by element.
   */
  public IOrderByElement getOrderByElement()
  {
    return orderByElement;
  }
  /**
  	Computes a hash code for the order by expression.
   */
  public int hashCode()
  {
    int hashCode = 0;

    if (operator != null)
      hashCode = operator.hashCode();

    if (orderByElement != null)
      hashCode = hashCode ^ orderByElement.hashCode();

    return hashCode;
  }
  /**
  	Sets the operator of the order by expression.
   */
  public void setOperator(IOperator newOperator)
  {
    operator = newOperator;
  }
  /**
  	Sets the order by element of the order by expression.
   */
  public void setOrderByElement(IOrderByElement newOrderByElement)
  {
    orderByElement = newOrderByElement;
  }
}
