package com.ibm.websphere.query.callbacks;

import java.util.Hashtable;
import java.util.Map;

import com.ibm.websphere.query.base.Constants;
import com.ibm.websphere.query.base.IValue;
import com.ibm.websphere.query.base.QueryException;

/**
 * Licensed Materials - Property of IBM. (C) COPYRIGHT International Business Machines Corp. 2002.
 */

/**
 * <P>The DB2400SqlSelectQueryCallback class is a query builder callback that
 * adds Timestamp formatting for AS/400 (iSeries)
 * */
public class DB2400SqlSelectQueryCallback extends DB2SqlSelectQueryCallback {

        /**
         * Constructor for DB2400SqlSelectQueryCallback.
         */
        public DB2400SqlSelectQueryCallback() {
                super();
        }

        /**
         * Constructor for DB2400SqlSelectQueryCallback.
         * @param newNameMapping
         */
        public DB2400SqlSelectQueryCallback(Hashtable newNameMapping) {
                super(newNameMapping);
        }

        /**
         * Constructor for DB2400SqlSelectQueryCallback.
         * @param newNameMapping
         * @param newTypeMapping
         */
        public DB2400SqlSelectQueryCallback(
                Map newNameMapping,
                Map newTypeMapping) {
                super(newNameMapping, newTypeMapping);
        }

        /**
                <P>Returns the SQL value string for the given value.
        
                <P>This method generates the value string in SQL based on the data type.
         */
        public void buildValue(IValue value, StringBuffer sb, Map metadata) throws QueryException {
                String s = value.getValueString();
                if (s.indexOf("'") != -1) {
                        s = makeQuerySafeValue(s);
                }

                int dataType = value.getDataType();
                if (typeMapping != null && dataType == Constants.DATATYPE_UNKNOWN) {
                        String propertyName = value.getPropertyName();
                        if (propertyName != null) {
                                Object o = typeMapping.get(propertyName);
                                if (o != null && o instanceof Integer) {
                                        dataType = ((Integer) o).intValue();
                                }
                        }
                }

                switch (dataType) {
                        case Constants.DATATYPE_BIGINT :
                        case Constants.DATATYPE_DECIMAL :
                        case Constants.DATATYPE_DOUBLE :
                        case Constants.DATATYPE_FLOAT :
                        case Constants.DATATYPE_TINYINT :
                        case Constants.DATATYPE_NUMERIC :
                        case Constants.DATATYPE_INTEGER :
                        case Constants.DATATYPE_SMALLINT :
                        case Constants.DATATYPE_REAL :
                        case Constants.DATATYPE_UNKNOWN :
                        case Constants.DATATYPE_NULL :
                                break;
                        case Constants.DATATYPE_CHAR :
                        case Constants.DATATYPE_LONGVARCHAR :
                        case Constants.DATATYPE_VARCHAR :
                        case Constants.DATATYPE_DATE :
                        case Constants.DATATYPE_TIME :
                                s = SQL_QUERY_STRINGSEP + s + SQL_QUERY_STRINGSEP;
                                break;
                        case Constants.DATATYPE_TIMESTAMP :
                                s = s.trim();
                                s = s.replace(':','.');
                                s = s.replace(' ','-');
                                s = SQL_QUERY_STRINGSEP + s + SQL_QUERY_STRINGSEP;
                                break;
                        default :
                                throw new QueryException("Data type not supported.", 3);
                }

                sb.append(s);
        }

}
