/* @copyright module */
/*                                                                            */
/*  DISCLAIMER OF WARRANTIES:                                                 */
/*  -------------------------                                                 */
/*  The following [enclosed] code is sample code created by IBM Corporation.  */
/*  This sample code is provided to you solely for the purpose of assisting   */
/*  you in the development of your applications.                              */
/*  The code is provided "AS IS", without warranty of any kind. IBM shall     */
/*  not be liable for any damages arising out of your use of the sample code, */
/*  even if they have been advised of the possibility of such damages.        */
package com.ibm.etools.portal.portletexamples.cooperative.legacy;

/*
 *
 * This portlet will get data from a mock database and 
 * display order detail of a order 
 *
 */ 
import java.io.IOException;
import java.text.MessageFormat;

import org.apache.jetspeed.portlet.PortletAdapter;
import org.apache.jetspeed.portlet.PortletException;
import org.apache.jetspeed.portlet.PortletRequest;
import org.apache.jetspeed.portlet.PortletResponse;
import org.apache.jetspeed.portlet.event.ActionEvent;
import org.apache.jetspeed.portlet.event.ActionListener;

import com.ibm.wps.logging.LogManager;
import com.ibm.wps.logging.Logger;


public class OrderDetailPortlet 
extends PortletAdapter
implements ActionListener {

    private static final String PREFIX = "THIS";
    public static final String ACTION_NAME = PREFIX + "actionName";
    public static final String ORDER_DETAILS = PREFIX + "orderDetails";
    public static final String ORDER_ID_ENTRY = PREFIX + "orderIdEntry";
    public static final String ORDER_ID = PREFIX + "orderId";
    public static final String ORDER_DETAIL_BEAN = PREFIX + "orderDetailBean";
    public static final String ORDER_DETAIL = PREFIX + "orderDetail";
    public static final String TRACKING_ID = PREFIX + "trackingId";

    private static Logger log = LogManager.getLogManager().getLogger(OrderDetailPortlet.class);

    /**
     * Process the portlet request, according to action 
     * to create protlet response
     *
     * @param request The Portlet request we are processing
     * @param response The Portlet response we are creating
     *
     * @exception PortletException,IOException
     */
    public void doView(PortletRequest request, PortletResponse response) 
    throws PortletException,IOException
    {
        if ( log.isLogging(Logger.TRACE_MEDIUM) ) {
            log.entry(Logger.TRACE_MEDIUM, "doView");
        }

        OrderDetailBean odb = new OrderDetailBean();
        try {   

            String action = (String) request.getPortletSession().getAttribute(ACTION_NAME);
            if ( log.isLogging(Logger.TRACE_MEDIUM) ) {
                log.text(Logger.TRACE_MEDIUM, "doView", "action = " + action);
            }

            request.setAttribute(ORDER_DETAIL_BEAN, odb);

            if (action == null || action.equals(ORDER_ID_ENTRY)) {

                //Show initial page
                String actionURI = ShippingUtils.createSimpleActionURI(ORDER_DETAILS, response);
                odb.setActionURI(actionURI);
                getPortletConfig().getContext().include("/jsp/html/OrderDetailEntry.jsp",
                                                        request, response);

            } else if (action.equals(ORDER_DETAILS)) {

                String orderId = (String) request.getPortletSession().getAttribute(ORDER_ID);
                if ( log.isLogging(Logger.TRACE_MEDIUM) ) {
                    log.text(Logger.TRACE_MEDIUM, "doView", "orderId = " + orderId);
                }

                OrderDetail od = ShippingDB.getOrderDetail(orderId);

                String actionURI = ShippingUtils.createSimpleActionURI(ORDER_DETAILS, response);

                odb.setActionURI(actionURI);

                if (od != null) {
                    odb.setOrderDetail(od);
                    getPortletConfig().getContext().include("/jsp/html/OrderDetailView.jsp",
                                                            request, response);
                } else {
                    //No match; print error page
                	String msg = MessageFormat.format(Messages.getString("ERR_OrderDetailPortlet.0"), new String[] {orderId}); //$NON-NLS-1$
                    odb.setErrorMessage(msg);
                    getPortletConfig().getContext().include("/jsp/html/OrderDetailError.jsp", request, response);
                }
            }
        } catch (Exception exc) {
            //Exception; print error page 
            request.setAttribute(ORDER_DETAIL_BEAN, odb);
            odb.setErrorMessage(Messages.getString("ERR_OrderDetailPortlet.1")); //$NON-NLS-1$
            getPortletConfig().getContext().include("/jsp/html/OrderDetailError.jsp", request, response);
            log.text(Logger.ERROR, "doView", "unexpected exception", exc);
        }
        if ( log.isLogging(Logger.TRACE_MEDIUM) ) {
            log.exit(Logger.TRACE_MEDIUM, "doView");
        }
    } 

    /**
     * Process ActionEvent, according to action name
     * to set request attribute
     *
     * @param event The ActionEvent we are processing
     *
     */  
    public void actionPerformed (ActionEvent event) 
    {
        if ( log.isLogging(Logger.TRACE_MEDIUM) ) {
            log.entry(Logger.TRACE_MEDIUM, "actionPerformed");
        }
        String actionName = event.getActionString();
        if ( log.isLogging(Logger.TRACE_MEDIUM) ) {
            log.text(Logger.TRACE_MEDIUM, "actionPerformed", "action = " + actionName);
        }
        PortletRequest request = event.getRequest();

        //An action causes the state to be modified
        ShippingUtils.setLastModified(request);


        if (actionName.equals(ORDER_DETAILS)) {
            request.getPortletSession().setAttribute(ACTION_NAME, ORDER_DETAILS);
            request.getPortletSession().setAttribute(ORDER_ID, request.getParameter(ORDER_ID));
            //We do this as tracking id is an out param in the C2A WSDL file
            //We write the tracking id in the request so it can be published by
            //the broker in the same event cycle
            String orderId = (String) request.getPortletSession().getAttribute(ORDER_ID);
            if ( log.isLogging(Logger.TRACE_MEDIUM) ) {
                log.text(Logger.TRACE_MEDIUM, "actionPerformed", "orderId = " + orderId);
            }
            OrderDetail od = ShippingDB.getOrderDetail(request.getParameter(ORDER_ID));
            request.getPortletSession().setAttribute(ORDER_DETAIL, od);
            if (od != null)
            request.setAttribute(TRACKING_ID, od.getTrackingId());

        } else if (actionName.equals(ORDER_ID_ENTRY)) {
            request.getPortletSession().setAttribute(ACTION_NAME, ORDER_ID_ENTRY);
        }
        if ( log.isLogging(Logger.TRACE_MEDIUM) ) {
            log.exit(Logger.TRACE_MEDIUM, "actionPerformed");
        }
    }

    public long getLastModified(PortletRequest request) {
        if ( log.isLogging(Logger.TRACE_MEDIUM) ) {
            log.entry(Logger.TRACE_MEDIUM, "getLastModified");
        }
        long lastModified = ShippingUtils.getLastModified(request);
        if ( log.isLogging(Logger.TRACE_MEDIUM) ) {
            log.exit(Logger.TRACE_MEDIUM, "getLastModified", lastModified);
        }
        return lastModified;
    }
}

