/* @copyright module */
/*                                                                            */
/*  DISCLAIMER OF WARRANTIES:                                                 */
/*  -------------------------                                                 */
/*  The following [enclosed] code is sample code created by IBM Corporation.  */
/*  This sample code is provided to you solely for the purpose of assisting   */
/*  you in the development of your applications.                              */
/*  The code is provided "AS IS", without warranty of any kind. IBM shall     */
/*  not be liable for any damages arising out of your use of the sample code, */
/*  even if they have been advised of the possibility of such damages.        */
package com.ibm.etools.portal.portletexamples.cooperative.jsr;

import java.io.IOException;
import java.util.Calendar;
import java.util.GregorianCalendar;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.GenericPortlet;
import javax.portlet.PortletConfig;
import javax.portlet.PortletException;
import javax.portlet.PortletRequest;
import javax.portlet.PortletResponse;
import javax.portlet.PortletURL;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;

import com.ibm.portal.portlet.service.PortletServiceHome;
import com.ibm.portal.propertybroker.service.PropertyBrokerService;

public class OrderMonthPortlet 
       extends GenericPortlet
{

    private static final String PREFIX = "SHIPPINGORDERS";
    public static final String ACTION_NAME = PREFIX + "actionName";
    public static final String ORDERS_FOR_MONTH = PREFIX + "ordersForMonth";
    public static final String MONTH_ENTRY = PREFIX + "monthEntry";
    public static final String ORDER_MONTH = "orderMonth";
    public static final String ORDER_ID = "orderId";
    public static final String CUSTOMER_ID = "customerId";
    public static final String ORDER_MONTH_BEAN = PREFIX + "orderMonthBean";
    public static final String MONTH = PREFIX + "month";
    public static final String DEFAULTMONTH = PREFIX + "defaultMonth";
    public static final String NLSFILENAME = "nls.shippingc2a";
    public static final String ACTION_NAME_PARAM = ShippingUtils.ACTION_NAME_PARAM;
    public static final String PBSERVICE = PREFIX + "pbService";
    public static final String PBSERVICEAVAILABLE = PREFIX + "pbServiceAvailable";
    private static final String[] monthString = {"January","February","March","April","May","June","July","August","September","October","November","December"};

	private static final String ENTRY_JSP = "OrderMonthEntry.jsp"; //$NON-NLS-1$
	private static final String VIEW_JSP = "OrderMonthView.jsp"; //$NON-NLS-1$
	private static final String ERROR_JSP = "OrderMonthError.jsp"; //$NON-NLS-1$

    boolean pbServiceAvailable = false;
    PropertyBrokerService pbService = null;
    public void init(PortletConfig config) throws PortletException {
        super.init(config);

       try {
            Context ctx = new InitialContext();
            PortletServiceHome serviceHome = (PortletServiceHome) ctx.lookup("portletservice/com.ibm.portal.propertybroker.service.PropertyBrokerService"); 
            pbService = (PropertyBrokerService)serviceHome.getPortletService(com.ibm.portal.propertybroker.service.PropertyBrokerService.class);
            pbServiceAvailable = true;
       }catch(Throwable t) {
            getPortletContext().log("OrderMonthPortlet could not find property broker service!");
		}
    }   

    public void beginPage(PortletRequest request,
			  PortletResponse response) {
    }

    public void endPage(PortletRequest request,
			PortletResponse response) {
    }

    public void doView(RenderRequest request, RenderResponse response)
        throws PortletException,IOException
    {
        getPortletContext().log("OrderMonthPortlet doView called");
		// Set the MIME type for the render response
		response.setContentType(request.getResponseContentType());
        String action = (String) request.getPortletSession().getAttribute(ACTION_NAME);
        getPortletContext().log("OrderMonthPortlet action = " + action);
	OrderMonthBean omb = new OrderMonthBean();
	request.setAttribute(ORDER_MONTH_BEAN, omb);
        if (pbServiceAvailable == true) {
            request.setAttribute(PBSERVICEAVAILABLE, "true");
            request.setAttribute(PBSERVICE, pbService);
        } else {
            request.setAttribute(PBSERVICEAVAILABLE, "false");
        }
                    
        if (action != null && action.equals(MONTH_ENTRY)) {  
	    //Show initial page
            PortletURL actionURL = ShippingUtils.createSimpleActionURL(ORDERS_FOR_MONTH, response);
            omb.setActionURL(actionURL);
            getPortletContext().getRequestDispatcher(ShippingUtils.getJspFilePath(request, ENTRY_JSP)).include(request, response);
	} else if (action == null || action.equals(ORDERS_FOR_MONTH)) { 
            String month = getMonth(request);
	    Order orderList[] = ShippingDB.getOrders(month);

            PortletURL actionURL = ShippingUtils.createSimpleActionURL(ORDERS_FOR_MONTH, response);
            omb.setActionURL(actionURL);
	    if (orderList != null) {
	        omb.setOrders(orderList);
	        omb.setMonth(ShippingDB.normalizeMonth(month));
                getPortletContext().getRequestDispatcher(ShippingUtils.getJspFilePath(request, VIEW_JSP)).include(request, response);
	    } else {
	        //Print error page
	        omb.setErrorMessage("No orders found for " + month + ".");
                getPortletContext().getRequestDispatcher(ShippingUtils.getJspFilePath(request, ERROR_JSP)).include(request, response);
	    }
        }
   }

   public void processAction (ActionRequest request, ActionResponse response) {
            getPortletContext().log("OrderMonthPortlet processAction entry");

            String actionName = request.getParameter(ACTION_NAME_PARAM);
            if (actionName == null) {
                    actionName = "";
            }

            if (actionName.equals(ORDERS_FOR_MONTH)) {
                    request.getPortletSession().setAttribute(ACTION_NAME, ORDERS_FOR_MONTH);
                    request.getPortletSession().setAttribute(MONTH, getMonth(request));
            } else if (actionName.equals(MONTH_ENTRY)) {
                    request.getPortletSession().setAttribute(ACTION_NAME, MONTH_ENTRY);
            }
   }

   /**
    * Check the request first to see if either the month or default month have been
    * set, then the session, and finally if not found then lookup the current month
    */
   private String getMonth(PortletRequest request) {
       String rv = "";

       rv = request.getParameter(MONTH);
       if ( isStringNullOrEmpty(rv) ) {
           rv = request.getParameter("defaultMonth");
           if ( isStringNullOrEmpty(rv) ) {
               rv = (String) request.getPortletSession().getAttribute(MONTH);
               if ( isStringNullOrEmpty(rv) ) {
                   Calendar calendar = new GregorianCalendar();
                   rv = monthString[ calendar.get(Calendar.MONTH) ];
               }
           }
       }
       getPortletContext().log("OrderMonthPortlet getMonth rv = " + rv);
       return rv;
   }

   /**
    * 
    */
   private boolean isStringNullOrEmpty(String str) {
       boolean rv = true;

       if (str != null && str.trim().length() > 0) {
           rv = false;
       }

       return rv;
   }
}

