/*
 * LockCapabilities.java
 *
 * This file is part of the specification for the
 * Java Content Repository API
 */
package javax.jcr.lock;


/**
 * Allows discovery of locking support.
 * <p>
 * <b>Level 2 only</b>
 * <p>
 * Returned by <code>{@link javax.jcr.Workspace#getLockCapabilities()}</code>. Provides information
 * about what locking features are supported by this implementation.
 *
 * @author Peeter Piegaze
 * @author Stefan Guggisberg
 */
public interface LockCapabilities {

    /**
     * Reports whether exclusive locks are supported by this implementation.
     *
     * @return <code>true</code> if exclusive locks are supported;
     * <code>false</code> otherwise.
     */
    public boolean isExclusiveLockSupported();

    /**
     * Reports whether shared locks are supported by this implementation.
     *
     * @return <code>true</code> if shared locks are supported;
     * <code>false</code> otherwise.
     */
    public boolean isSharedLockSupported();

    /**
     * Reports whether locking at the the level of individual properties is
     * supported by this implementation.
     *
     * @return <code>true</code> if property locking is supported;
     * <code>false</code> otherwise.
     */
    public boolean isPropertyLockSupported();

    /**
     * Reports the supported lock types. If locking is supported at all, this
     * will include at least <code>{@link LockType#WRITE_LOCK}</code>. If
     * other lock types are also supported then these are returned as well.
     *
     * @return An array of <code>LockType</code> objects representing all
     * supported lock types.
     */
    public LockType[] getSupportedLockTypes();
}
