/*
 * ConstraintViolationException.java
 *
 * This file is part of the specification for the
 * Java Content Repository API
 */
package javax.jcr.nodetype;

import javax.jcr.RepositoryException;

/**
 * Exception thrown by methods of <code>Node</code> when
 * a constraint on repository structure is violated. For example,
 * (in level 1 and 2) when an attempt is made to move an item to a position
 * as a child of a property or (in level 2 only) when an attempt is
 * made to add an el;emwent ot a node that would violate that node's object
 * class.
 *
 * @author Peeter Piegaze
 */
public class ConstraintViolationException extends RepositoryException {

    /**
     * Class constructor that creates a <code>ConstraintViolationException</code>
     */
    public ConstraintViolationException() {
        super();
    }

    /**
     * Class constructor that creates a <code>ConstraintViolationException</code> given
     * a message describing the failure cause
     * @param s description
     */
    public ConstraintViolationException(String s) {
        super(s);
    }

    /**
     * Class constructor that creates a <code>ConstraintViolationException</code> given
     * a message describing the failure and a root exception
     * @param s description
     * @param e root failure cause
     */
    public ConstraintViolationException(String s, Exception e) {
        super(s, e);
    }
}
