/*
 * ChangeEvent.java
 *
 * This file is part of the specification for the
 * Java Content Repository API
 */
package javax.jcr.observation;

import javax.jcr.Item;

/**
 * A subclass of <code>Event</code> representing a "change" event.
 * <p>
 * <b>Level 2 only</b>
 * <p>
 * Will always be one of the following types:
 * <ul>
 * <li><code>EventType.ITEM_ADDED</code>
 * <li><code>EventType.ITEM_CHANGED</code>
 * <li><code>EventType.ITEM_REMOVED</code>
 * </ul>
 * For details see the <i>ObservationManager</i> section of the JCR standard document.
 *
 * @author tima, ppiegaze
 */
public interface ChangeEvent extends Event {

    /**
     * If this event is of <code>EventType.ITEM_ADDED</code> then
     * this method returns <code>null</code>.
     * <p>
     * If this event is of <code>EventType.ITEM_CHANGED</code> then this
     * method returns a copy of the item in question as it was <i>before</i>
     * the change.
     * <p>
     * If this event is of <code>EventType.ITEM_REMOVED</code> then this
     * method will return the removed item (as will the method
     * <code>Event.getItem()</code>, that is, they both return the same
     * item).
     *
     * @return a copy of the item as it was before the change or the removed
     * item if this is an <code>EventType.ITEM_REMOVED</code>.
     */
    public Item getOldItem();
}
