/*
 * LockEvent.java
 *
 * This file is part of the specification for the
 * Java Content Repository API
 */
package javax.jcr.observation;

import javax.jcr.lock.Lock;

/**
 * A subclass of <code>Event</code> representing a "locking" event.
 * <p>
 * <b>Level 2 only</b></code>
 * <p>
 * Will always
 * be one of the following types:
 * <ul>
 * <li><code>EventType.ITEM_LOCKED</code>
 * <li><code>EventType.ITEM_UNLOCKED</code>
 * <li><code>EventType.LOCK_EXPIRED</code>
 * </ul>
 * For details see the <i>ObservationManager</i> section of the JCR standard document.
 *
 * @author tima, ppiegaze
 */
public interface LockEvent extends Event {

    /**
     * Returns the lock object associated with the locking event.
     * @return a <code>Lock</code> object.
     */
    public Lock getLock();
}
