
/*
 * OnParentVersionAction.java
 *
 * This file is part of the specification for the
 * Java Content Repository API
 */
package javax.jcr.version;

public final class OnParentVersionAction {

    /**
     * The <code>OnParentVersionAction</code> action <code>COPY</copy>.
     */
    public static final int COPY = 1;

    /**
     * The <code>OnParentVersionAction</code> action <code>VERSION</copy>.
     */
    public static final int VERSION = 2;
    
    /**
     * The <code>OnParentVersionAction</code> action <code>INITIALIZE</copy>.
     */
    public static final int INITIALIZE = 3;
    
    /**
     * The <code>OnParentVersionAction</code> action <code>COMPUTE</copy>.
     */
    public static final int COMPUTE = 4;
    
    /**
     * The <code>OnParentVersionAction</code> action <code>IGNORE</copy>.
     */
    public static final int IGNORE = 5; 
    
    /**
     * The <code>OnParentVersionAction</code> action <code>ABORT</copy>.
     */
    public static final int ABORT = 6;


    /**
     * The names of the defined on-version actions,
     * as used in serialization.
     */
    public static final String ACTIONNAME_COPY = "COPY";
    public static final String ACTIONNAME_VERSION = "VERSION";
    public static final String ACTIONNAME_INITIALIZE = "INITIALIZE";
    public static final String ACTIONNAME_COMPUTE = "COMPUTE";
    public static final String ACTIONNAME_IGNORE = "IGNORE";
    public static final String ACTIONNAME_ABORT = "ABORT";

    /**
     * Returns the name of the specified <code>action</code>,
     * as used in serialization.
     * @param action the on-version action
     * @return the name of the specified <code>action</code>
     * @throws IllegalArgumentException if <code>action</code>
     * is not a valid on-version action.
     */
    public static String nameFromValue(int action) {
        switch (action) {
            case COPY:
                return ACTIONNAME_COPY;
            case VERSION:
                return ACTIONNAME_VERSION;
            case INITIALIZE:
                return ACTIONNAME_INITIALIZE;
            case COMPUTE:
                return ACTIONNAME_COMPUTE;
            case IGNORE:
                return ACTIONNAME_IGNORE;
            case ABORT:
                return ACTIONNAME_ABORT;
            default:
                throw new IllegalArgumentException("unknown on-version action: " + action);
        }
    }

    /**
     * Returns the numeric constant value of the on-version action with the
     * specified name.
     * @param name the name of the on-version action
     * @return the numeric constant value
     * @throws IllegalArgumentException if <code>name</code>
     * is not a valid on-version action name.
     */
    public static int valueFromName(String name) {
        if (name.equals(ACTIONNAME_COPY)) {
            return COPY;
        } else if (name.equals(ACTIONNAME_VERSION)) {
            return VERSION;
        } else if (name.equals(ACTIONNAME_INITIALIZE)) {
            return INITIALIZE;
        } else if (name.equals(ACTIONNAME_COMPUTE)) {
            return COMPUTE;
        } else if (name.equals(ACTIONNAME_IGNORE)) {
            return IGNORE;
        } else if (name.equals(ACTIONNAME_ABORT)) {
            return ABORT;
        } else {
            throw new IllegalArgumentException("unknown on-version action: " + name);
        }
    }

    private OnParentVersionAction() {
    }

}
